﻿Shader "Hidden/Bloom"
{
	Properties
	{
		_MainTex ("Texture", 2D) = "white" {}
		_MainTexFull ("Texture (Full scale)", 2D) = "white" {}
	}
	SubShader
	{
		// No culling or depth
		Cull Off ZWrite Off ZTest Always

		Pass
		{
			CGPROGRAM
			#pragma vertex vert
			#pragma fragment frag
			
			#include "UnityCG.cginc"

			struct appdata
			{
				float4 vertex : POSITION;
				float2 uv : TEXCOORD0;
			};

			struct v2f
			{
				float2 uv : TEXCOORD0;
				float4 vertex : SV_POSITION;
			};

			v2f vert (appdata v)
			{
				v2f o;
				o.vertex = UnityObjectToClipPos(v.vertex);
				o.uv = v.uv;
				return o;
			}
			
			sampler2D _MainTex;
			sampler2D _MainTexFull;
			float4 _MainTex_TexelSize;

			fixed4 box(float2 uv){
				fixed4 colSum = (
					tex2D(_MainTex, uv + float2(_MainTex_TexelSize.x, _MainTex_TexelSize.y)) +
					tex2D(_MainTex, uv + float2(0, _MainTex_TexelSize.y)) +
					tex2D(_MainTex, uv + float2(_MainTex_TexelSize.x * -1, _MainTex_TexelSize.y)) +

					tex2D(_MainTex, uv + float2(_MainTex_TexelSize.x, 0)) +
					tex2D(_MainTex, uv + float2(0, 0)) +
					tex2D(_MainTex, uv + float2(_MainTex_TexelSize.x * -1, 0)) +

					tex2D(_MainTex, uv + float2(_MainTex_TexelSize.x, _MainTex_TexelSize.y * -1)) +
					tex2D(_MainTex, uv + float2(0, _MainTex_TexelSize.y * -1)) +
					tex2D(_MainTex, uv + float2(_MainTex_TexelSize.x * -1, _MainTex_TexelSize.y * -1))
				);

				return colSum / 9;
			}

			fixed4 frag (v2f i) : SV_Target
			{
				fixed4 col = box(i.uv);
				fixed4 col2 = tex2D(_MainTexFull, i.uv);
				return lerp(col, col2, 0.4);
			}
			ENDCG
		}
	}
}
